Module queries

Module queries 

Queries needed for polling, updating and exposing tasks.

Functions

fetch_next
Fetch the next batch of tasks from the sqlite backend
handle_results
Ack multiple tasks, given a worker
keep_alive
Heartbeat for denoting liveliness of workers
lock_tasks
Lock multiple tasks, given a worker
push_tasks
Push a batch of tasks to the database
reenqueue_abandoned
Rescues tasks that could not be executed after a worker shutdown
reenqueue_orphaned
Reenqueue jobs orphaned by a dead worker
register_worker
Register a worker in the database

Type Aliases

ResultPayload
Serialized result payload including the current attempt and status