pub struct PostgresStorage<Args> { /* private fields */ }Expand description
A backend for persisting and consuming jobs behind a postgres database
Features
| Feature | Status | Description |
|---|---|---|
Backend | ✅ | Supports storage and retrieval of tasks |
TaskSink | ✅ | Ability to push new tasks |
Serialization | ✅ | Serialization support for arguments |
Workflow | ✅ | Flexible enough to support workflows |
Web Interface | ✅ | Expose a web interface for monitoring tasks |
FetchById | ✅ | Allow fetching a task by its ID |
RegisterWorker | ✅ | Allow registering a worker with the backend |
MakeShared | ✅ | Share one connection across multiple workers via PostgresStorageFactory |
WaitForCompletion | ✅ | Wait for tasks to complete without blocking |
ResumeById | ✅ | Resume a task by its ID |
ResumeAbandoned | ✅ | Resume abandoned tasks |
ListWorkers | ✅ | List all workers registered with the backend |
ListTasks | ✅ | List all tasks in the backend |
Key: ✅ : Supported | ⚠️ : Not implemented | ❌ : Not Supported | ❗ Limited support
Tests:
Backend
#[tokio::main]
async fn main() {
// let mut backend = /* snip */;
async fn task(task: u32, worker: WorkerContext) {
// Do some work
}
let worker = WorkerBuilder::new("backend-test")
.backend(backend)
.build(task);
let _ = worker.stream().take(1).collect::<Vec<_>>().await;
}
TaskSink
#[tokio::main]
async fn main() {
// let mut backend = /* snip */;
backend.push(42).await.unwrap();
async fn task(task: u32, worker: WorkerContext) {
worker.stop().unwrap();
}
let worker = WorkerBuilder::new("task-sink-test")
.backend(backend)
.build(task);
worker.run().await.unwrap();
}
Serialization
#[tokio::main]
async fn main() {
// let mut backend = /* snip */;
fn assert_codec<B: BackendConfig + WireFormatBackend>(backend: B)
where
B::Codec: Codec<B::Args, Compact=Vec<u8>>,
{
}
assert_codec(backend);
}
Workflow
#[tokio::main]
async fn main() {
// let mut backend = /* snip */;
backend.push(42).await.unwrap();
async fn task1(task: u32, worker: WorkerContext) -> u32 {
task + 99
}
async fn task2(task: u32, worker: WorkerContext) -> u32 {
task + 1
}
async fn task3(task: u32, worker: WorkerContext) {
assert_eq!(task, 142);
worker.stop().unwrap();
}
let workflow = SteppedFlow::new("test-workflow")
.and_then(task1)
.and_then(task2)
.and_then(task3);
let worker = WorkerBuilder::new("workflow-test")
.backend(backend)
.build(workflow);
worker.run().await.unwrap();
}
WebUI
#[tokio::main]
async fn main() {
// let mut backend = /* snip */;
fn assert_web_ui<B: Expose<u32, Durable>>(backend: B) {};
assert_web_ui(backend);
}
WaitForCompletion
#[tokio::main]
async fn main() {
// let mut backend = /* snip */;
fn assert_wait_for_completion<B: WaitForCompletion<u32> + Backend>(backend: B) {};
assert_wait_for_completion(backend);
}
Implementations
Sourceimpl PostgresStorage<()>
impl PostgresStorage<()>
Sourcepub async fn setup(pool: &PgPool) -> Result<(), Error>
pub async fn setup(pool: &PgPool) -> Result<(), Error>
Runs the PostgreSQL storage migrations.
Fresh databases
No manual setup is required. Calling setup() will create the required
tables and migration history.
Upgrading to 1.0
⚠️ Important: Existing databases created by a pre-
1.0version require a one-time manual migration before callingsetup().
The 1.0 migration history is no longer relocated automatically by
setup(). Follow the “Upgrading to 1.0” section in the README to
perform the required transition.
After the transition has been completed, setup() can be used normally
for subsequent migrations.
Example
use apalis_postgres::PostgresStorage;
use sqlx::PgPool;
PostgresStorage::<()>::setup(&pool).await?;Errors
Returns an error if the migrations cannot be applied to the database.
Sourcepub fn migrations() -> Migrator
pub fn migrations() -> Migrator
Get postgres migrations without running them
Sourceimpl<Args> PostgresStorage<Args>
impl<Args> PostgresStorage<Args>
Sourcepub fn with_pubsub(self) -> PollWith<Self, StreamStrategy<Pubsub>>
pub fn with_pubsub(self) -> PollWith<Self, StreamStrategy<Pubsub>>
Mount a standalone Pubsub which uses its own connection under the hood
Sourcepub fn with_config(self, config: Config) -> Self
pub fn with_config(self, config: Config) -> Self
Configure a new PostgresStorage instance.