Queries needed for polling, updating and exposing tasks.
Functions
- fetch_
next - Fetch the next batch of tasks from the sqlite backend
- handle_
results - Ack multiple tasks, given a worker
- keep_
alive - Heartbeat for denoting liveliness of workers
- lock_
tasks - Lock multiple tasks, given a worker
- push_
tasks - Push a batch of tasks to the database
- reenqueue_
abandoned - Rescues tasks that could not be executed after a worker shutdown
- reenqueue_
orphaned - Reenqueue jobs orphaned by a dead worker
- register_
worker - Register a worker in the database
Type Aliases
- Result
Payload - Serialized result payload including the current attempt and status