PostgresStorage

Struct PostgresStorage 

pub struct PostgresStorage<Args> { /* private fields */ }
A backend for persisting and consuming jobs behind a postgres database

Features

FeatureStatusDescription
BackendSupports storage and retrieval of tasks
TaskSinkAbility to push new tasks
SerializationSerialization support for arguments
WorkflowFlexible enough to support workflows
Web InterfaceExpose a web interface for monitoring tasks
FetchByIdAllow fetching a task by its ID
RegisterWorkerAllow registering a worker with the backend
MakeSharedShare one connection across multiple workers via PostgresStorageFactory
WaitForCompletionWait for tasks to complete without blocking
ResumeByIdResume a task by its ID
ResumeAbandonedResume abandoned tasks
ListWorkersList all workers registered with the backend
ListTasksList all tasks in the backend

Key: ✅ : Supported | ⚠️ : Not implemented | ❌ : Not Supported | ❗ Limited support

Tests:
Backend
#[tokio::main]
async fn main() {
    // let mut backend = /* snip */;
    
    async fn task(task: u32, worker: WorkerContext) {
        // Do some work 
    }
    let worker = WorkerBuilder::new("backend-test")
        .backend(backend)
        .build(task);
    let _ = worker.stream().take(1).collect::<Vec<_>>().await;
}
TaskSink
#[tokio::main]
async fn main() {
    // let mut backend = /* snip */;
    
    backend.push(42).await.unwrap();

    async fn task(task: u32, worker: WorkerContext) {
        worker.stop().unwrap();
    }
    let worker = WorkerBuilder::new("task-sink-test")
        .backend(backend)
        .build(task);
    worker.run().await.unwrap();
}
Serialization
#[tokio::main]
async fn main() {
    // let mut backend = /* snip */;
    
   fn assert_codec<B: BackendConfig + WireFormatBackend>(backend: B) 
   where
       B::Codec: Codec<B::Args, Compact=Vec<u8>>,
   {
   }
   assert_codec(backend);
}
Workflow
#[tokio::main]
async fn main() {
    // let mut backend = /* snip */;
    
    backend.push(42).await.unwrap();

    async fn task1(task: u32, worker: WorkerContext) -> u32 {
        task + 99 
    }
    async fn task2(task: u32, worker: WorkerContext) -> u32 {
        task + 1 
    }
    async fn task3(task: u32, worker: WorkerContext) {
        assert_eq!(task, 142);
        worker.stop().unwrap();
    }
    let workflow = SteppedFlow::new("test-workflow")
       .and_then(task1)
       .and_then(task2)
       .and_then(task3);
    let worker = WorkerBuilder::new("workflow-test")
        .backend(backend)
        .build(workflow);
    worker.run().await.unwrap();
}
WebUI
#[tokio::main]
async fn main() {
    // let mut backend = /* snip */;
    
    fn assert_web_ui<B: Expose<u32, Durable>>(backend: B) {};
    assert_web_ui(backend);
}
WaitForCompletion
#[tokio::main]
async fn main() {
    // let mut backend = /* snip */;
    
    fn assert_wait_for_completion<B: WaitForCompletion<u32> + Backend>(backend: B) {};
    assert_wait_for_completion(backend);
}

Implementations

Source

impl PostgresStorage<()>

pub async fn setup(pool: &PgPool) -> Result<(), Error>

Runs the PostgreSQL storage migrations.

Fresh databases

No manual setup is required. Calling setup() will create the required tables and migration history.

Upgrading to 1.0

⚠️ Important: Existing databases created by a pre-1.0 version require a one-time manual migration before calling setup().

The 1.0 migration history is no longer relocated automatically by setup(). Follow the “Upgrading to 1.0” section in the README to perform the required transition.

After the transition has been completed, setup() can be used normally for subsequent migrations.

Example
use apalis_postgres::PostgresStorage;
use sqlx::PgPool;

PostgresStorage::<()>::setup(&pool).await?;
Errors

Returns an error if the migrations cannot be applied to the database.

pub fn migrations() -> Migrator

Get postgres migrations without running them

Source

impl<Args> PostgresStorage<Args>

pub fn new(pool: &PgPool) -> Self

Creates a new PostgresStorage instance.

pub fn with_pubsub(self) -> PollWith<Self, StreamStrategy<Pubsub>>

Mount a standalone Pubsub which uses its own connection under the hood

pub fn with_config(self, config: Config) -> Self

Configure a new PostgresStorage instance.

pub fn pool(&self) -> &PgPool

Returns a reference to the pool.

pub fn config(&self) -> &Config

Returns a reference to the config.